% Created 2012-August-20
% Written by Fu Lin
% Choose noise-corrupted leaders using degree heuristics.
% Inputs: 
%        L     -> the Laplacian matrix of the graph;
%        kappa -> the vector of positive elements to strengthen the diagonal of L;
%        k     -> the number of leaders.
% Outputs:
%         Jval       -> the objective function values;
%         leader_idx -> the indices of the leaders.

function [Jval,leader_idx] = degree_noise_corrupted_leader(L,kappa,k)

    degree = diag(L);    
    [~,sort_idx] = sort(degree,'descend');
    leader_idx = sort_idx(1:k);
    Jval = zeros(k,1);
    
    Da = diag(kappa);
    for i = 1:k
        v = zeros(size(L,1),1);
        v(leader_idx(1:i)) = 1;
        Jval(i) = trace(inv(L + Da.*diag(v)));
    end
    
end