% Created 2012-April-2
% Written by Fu Lin
% Draw communication graphs and the leaders.
% inputs: 
% (1) position information
% (2) indices of neighboring nodes
% (3) indices of leaders
% output: figure of the connected network with leaders

function drawleaders(pos,idx,pos_leaders)

% plot the graph where links indicate neighbors
figure
hold on

% perturb the positions a little bit
% to make sure the points are not exactly located at the x or y axes 
% otherwise the plot command may plot the links incorrectly
pos = pos + 1.e-10;

% draw links
ii = sqrt(-1);
for k = 1 : max(size(idx))
    i = idx(1,k);
    j = idx(2,k);    
    plot( [pos(i,1) + ii*pos(i,2), pos(j,1) + ii*pos(j,2)], ':k', 'LineWidth', 2);
end

% draw nodes
plot( pos(:,1) + ii*pos(:,2),'o','LineWidth',2,'MarkerSize',10)

% draw leaders
plot( pos_leaders(:,1) + ii*pos_leaders(:,2),'o','LineWidth',2,...
    'MarkerSize',10,'MarkerFaceColor','r')

hold off;
hh = get(gcf,'CurrentAxes');
set(hh, 'FontName', 'cmr10', 'FontSize', 18)    

